""" mcdutils proxy

$Id: proxy.py,v 1.7 2006/05/31 20:57:16 tseaver Exp $
"""
import memcache
import UserDict

from zope.interface import implementedBy
from zope.interface import implements

from AccessControl.SecurityInfo import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.mcdutils.interfaces import IMemCacheProxy
from Products.mcdutils.mapping import MemCacheMapping

class FauxClient(UserDict.UserDict):

    def _get_server(self, key):
        return self, key

    def set(self, key, value):
        self[key] = value

class MemCacheProxy(SimpleItem, PropertyManager):
    """ Implement ISDC via a a pool of memcache servers.
    """
    implements(IMemCacheProxy
             + implementedBy(SimpleItem)
             + implementedBy(PropertyManager)
             )
    security = ClassSecurityInfo()

    _v_cached = None
    _v_client = None

    security.declarePrivate('_get_cached')
    def _get_cached(self):
        if self._v_cached is None:
            self._v_cached = {}
        return self._v_cached

    _cached = property(_get_cached,)

    security.declarePrivate('_get_client')
    def _get_client(self):
        if self._v_client is not None:
            return self._v_client

        if self.servers:
            client = self._v_client = memcache.Client(self.servers)
        else:
            client = self._v_client = FauxClient()

        return client

    client = property(_get_client,)

    _servers = ()

    security.declarePrivate('_set_servers')
    def _set_servers(self, value):
        self._servers = value
        try:
            del self._v_client
        except AttributeError:
            pass
        try:
            del self._v_cache
        except AttributeError:
            pass

    servers = property(lambda self: self._servers, _set_servers)

    #
    #   ZMI
    #
    meta_type = 'MemCache Proxy'
    _properties = (
        {'id': 'servers', 'type': 'lines', 'mode': 'w'},
    )

    manage_options = (PropertyManager.manage_options
                    + SimpleItem.manage_options
                     )

    security.declarePrivate('get')
    def get(self, key):
        """ See IMemCacheProxy.
        """
        mapping = self._cached.get(key)

        if mapping is None:
            mapping = self._get_remote(key)

        return mapping

    security.declarePrivate('create')
    def create(self, key):
        """ See IMemCacheProxy.
        """
        mapping = self._cached[key] = MemCacheMapping(key, self)
        return mapping

    security.declarePrivate('set')
    def set(self, key, value):
        """ See IMemCacheProxy.
        """
        self.client.set(key, value.data)
        try:
            del self._cached[key]
        except KeyError:
            pass

    #
    #   Helper methods
    #
    security.declarePrivate('_get_remote')
    def _get_remote(self, key):
        data = self.client.get(key)
        if data is None:
            return None
        mapping = self.create(key)
        mapping.data.update(data)
        return mapping

InitializeClass(MemCacheProxy)


def addMemCacheProxy(dispatcher, id, REQUEST):
    """ Add a MCP to dispatcher.
    """
    proxy = MemCacheProxy()
    proxy._setId(id)
    dispatcher._setObject(id, proxy)
    REQUEST['RESPONSE'].redirect('%s/manage_workspace'
                                    % dispatcher.absolute_url())

addMemCacheProxyForm = PageTemplateFile('www/add_mcp.pt', globals())
